library in_button;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Button Input Module for Lightning.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2011
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

 {$IMAGEBASE $41030000}

uses
  Windows,
  SysUtils,
  Controls,
  StdCtrls,
  ExtCtrls,
  Graphics,
  Classes,
  Messages,
  common in '..\common.pas',
  unitkeyedit in '..\unitkeyedit.pas';

{$R *.RES}
{$R ..\RESOURCES\BUTTONS.RES}

type
  TDetails = class(TObject)
  private
    bitmapButtonDown, bitmapButtonUp: TBitmap;
    pButton: TPanel;
    iButton: TImage;
    lButton: TLabel;
    pressed: Boolean;
    key: Longint;
    keyShift, keyControl, keyScrollLock: Boolean;
    procedure setPressed(pressed: Boolean);
    procedure setKey(key: Longint; keyShift: Boolean; keyControl: Boolean; keyScrollLock: Boolean);
    function getKey(): Longint;
  public
    gbDetails: TGroupBox;
    labelKey: TLabel;
    editKey: TKeyEdit;
    checkBoxScrollLock: TCheckBox;
    checkBoxShift: TCheckBox;
    checkBoxControl: TCheckBox;
    procedure iButtonMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure iButtonMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure keyShiftControl(Sender: TObject; shift: Boolean; control: Boolean);
    procedure keyChange(Sender: TObject);
  end;

procedure TDetails.iButtonMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  setPressed(True);
end;

procedure TDetails.iButtonMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  setPressed(False);
end;

procedure TDetails.keyShiftControl(Sender: TObject; shift: Boolean; control: Boolean);
begin
  checkBoxShift.OnClick   := nil;
  checkBoxControl.OnClick := nil;
  try
    checkBoxShift.Checked   := shift;
    checkBoxControl.Checked := control;
  finally
    checkBoxShift.OnClick   := keyChange;
    checkBoxControl.OnClick := keyChange;
  end;
end;

procedure TDetails.keyChange(Sender: TObject);
begin
  setKey(
    editKey.getVirtualKey(),
    checkBoxShift.Checked,
    checkBoxControl.Checked,
    checkBoxScrollLock.Checked
  );
end;

procedure TDetails.setPressed(pressed: Boolean);
begin
  if Self.pressed <> pressed then
  begin
    Self.pressed := pressed;
    if pressed then
    begin
      iButton.Picture.Assign(bitmapButtonDown);
      lButton.Top  := lButton.Top + 1;
      lButton.Left := lButton.Left + 1;
    end
    else
    begin
      iButton.Picture.Assign(bitmapButtonUp);
      lButton.Top  := lButton.Top - 1;
      lButton.Left := lButton.Left - 1;
    end;
  end;
end;

procedure TDetails.setKey(key: Longint; keyShift: Boolean; keyControl: Boolean; keyScrollLock: Boolean);
var
  buf: string;
begin
  Self.key           := key;
  Self.keyShift      := keyShift;
  Self.keyControl    := keyControl;
  Self.keyScrollLock := keyScrollLock;
  buf := TKeyEdit.translateKey(key);
  // only show hotkey in module's button if it's 1 character long and
  // requires no modifiers (Shift, Control).
  if not(keyShift)
     and
     not(keyControl)
     and
     (Length(buf) = 1) then
  begin
    if keyScrollLock then
      // show hotkey in red if scroll lock must be pressed
      lButton.Font.Color := clRed
    else
      // show hotkey in yellow if scroll lock must be depressed
      lButton.Font.Color := clYellow;
    lButton.Caption := buf;
    lButton.Visible := True;
  end
  else
  begin
    lButton.Caption := '';
    lButton.Visible := False;
  end;
end;

function TDetails.getKey(): Longint;
begin
  Result := key;
end;

function init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  Result := return('IButton|Controls', name, nameLength);
end;

procedure finish(); cdecl;
begin
  //
end;

function create(controlHandle: THandle): PLightningModule; cdecl;
var
  myDetails: TDetails;
begin
  New(Result);
  with Result^ do
  begin
    handle        := controlHandle;
    title         := 'Button';
    settings      := True;
    width         := 56;
    height        := -1;
    inputTop      := -1;
    infoTop       := -1;
    outputTop     := -1;
    inputs        := 0;
    outputs       := 1;
    inputTooltip  := '';
    outputTooltip := 'Button status';
    myDetails     := TDetails.Create;
    reserved      := nil;
    details       := myDetails;
  end;
  with myDetails do
  begin
    pressed := False;
    bitmapButtonUp := TBitmap.Create();
    with bitmapButtonUp do
    begin
      LoadFromResourceName(hInstance, 'buttonup');
    end;
    bitmapButtonDown := TBitmap.Create();
    with bitmapButtonDown do
    begin
      LoadFromResourceName(hInstance, 'buttondown');
    end;
    pButton := TPanel.Create(nil);
    with pButton do
    begin
      Top        := 32;
      Left       := 8;
      BevelOuter := bvNone;
    end;
    iButton := TImage.Create(pButton);
    with iButton do
    begin
      Cursor      := crHandPoint;
      AutoSize    := True;
      Picture.Assign(bitmapButtonUp);
      OnMouseDown := iButtonMouseDown;
      OnMouseUp   := iButtonMouseUp;
      Parent      := pButton;
    end;
    lButton := TLabel.Create(pButton);
    with lButton do
    begin
      Transparent := True;
      Cursor      := crHandPoint;
      AutoSize    := False;
      Top         := 0;
      Left        := 0;
      Width       := iButton.Width;
      Height      := iButton.Height;
      Alignment   := taCenter;
      Layout      := tlCenter;
      with Font do
      begin
        Name  := 'Arial';
        Size  := 7;
        Color := clYellow;
      end;
      Caption     := '';
      OnMouseDown := iButton.OnMouseDown;
      OnMouseUp   := iButton.OnMouseUp;
      Visible     := False;
      Parent      := pButton;
    end;
    with pButton do
    begin
      Width        := iButton.Width;
      Height       := iButton.Height;
      ParentWindow := controlHandle;
    end;
  end;
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
begin
  with TDetails(lightningModule^.details) do
  begin
    bitmapButtonUp.Free();
    bitmapButtonDown.Free();
    iButton.Free();
    lButton.Free();
    pButton.Free();
    Free();
  end;
  Dispose(lightningModule);
end;

function action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
begin
  Result := True;
  if TDetails(lightningModule.details).pressed then
    createSpectrum(bridge, levelMax)
  else
    createSpectrum(bridge, levelMin);
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
begin
  with TDetails(lightningModule^.details) do
  begin
    gbDetails := TGroupBox.Create(nil);
    with gbDetails do
    begin
      Top     := detailsTop;
      Left    := detailsLeft;
      Width   := detailsWidth;
      Height  := detailsHeight;
      Ctl3D   := True;
      Color   := clBtnFace;
      Caption := 'Button Trigger Settings';
    end;
    checkBoxShift := TCheckBox.Create(gbDetails);
    with checkBoxShift do
    begin
      Top     := 20;
      Left    := 8;
      Width   := 60;
      Caption := 'Shift +';
      Checked := keyShift;
      OnClick := keyChange;
      Parent  := gbDetails;
    end;
    checkBoxControl := TCheckBox.Create(gbDetails);
    with checkBoxControl do
    begin
      Top     := 40;
      Left    := 8;
      Width   := 60;
      Caption := 'Ctrl +';
      Checked := keyControl;
      OnClick := keyChange;
      Parent  := gbDetails;
    end;
    labelKey := TLabel.Create(gbDetails);
    with labelKey do
    begin
      AutoSize := True;
      Top      := 20;
      Left     := 68;
      Caption  := 'Key:';
      Parent   := gbDetails;
    end;
    editKey := TKeyEdit.Create(gbDetails);
    with editKey do
    begin
      Top            := 40;
      Left           := 68;
      Width          := 100;
      ReadOnly       := True;
      Tag            := Longint(TDetails(lightningModule^.details));
      setVirtualKey(getKey());
      OnShiftControl := keyShiftControl;
      OnChange       := keyChange;
      Parent         := gbDetails;
    end;
    checkBoxScrollLock := TCheckBox.Create(gbDetails);
    with checkBoxScrollLock do
    begin
      Top       := 40;
      Left      := 178;
      Width     := 130;
      Caption   := '+ Scroll Lock enabled';
      Alignment := taLeftJustify;
      Checked   := keyScrollLock;
      OnClick   := keyChange;
      Parent    := gbDetails;
    end;
    gbDetails.ParentWindow := handle;
  end;
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  with TDetails(lightningModule^.details) do
  begin
    checkBoxShift.Free();
    checkBoxControl.Free();
    labelKey.Free();
    editKey.Free();
    checkBoxScrollLock.Free();
    gbDetails.Free();
  end;
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
var
  details: TDetails;
  key: Longint;
  bString, flags: string;
begin
  details := TDetails(lightningModule^.details);
  bString := settings;
  parse(bString, ','); // a label was stored here previously
  key := StrToIntDef(parse(bString, ','), 0);
  flags := parse(bString, ',');
  details.setKey(
    key,
    Pos('S', flags) > 0,
    Pos('C', flags) > 0,
    Pos('*', flags) > 0
  );
end;

function save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
var
  flags: string;
begin
  with TDetails(lightningModule^.details) do
  begin
    flags := '';
    if keyShift then
      flags := flags + 'S';
    if keyControl then
      flags := flags + 'C';
    if keyScrollLock then
      flags := flags + '*';
    Result := return(',' + IntToStr(getKey()) + ',' + flags, settings, settingsLength);
  end;
end;

procedure keyboard(lightningModule: PLightningModule; nVirtKey: LongInt; lKeyData: Cardinal); cdecl;
var
  details: TDetails;
begin
  details := TDetails(lightningModule^.details);
  if (nVirtKey = details.getKey()) then
  begin
    // check if this is a key up to down transition
    if (details.keyShift = (HIBYTE(GetKeyState(VK_SHIFT)) > 0))
       and
       (details.keyControl = (HIBYTE(GetKeyState(VK_CONTROL)) > 0))
       and
       (details.keyScrollLock = (LOBYTE(GetKeyState(VK_SCROLL)) > 0))
       and
       ((lKeyData and $40000000) = 0)
       and
       ((lKeyData and $80000000) = 0) then
    begin
      details.setPressed(True);
    end
    // check if this is a key down to up transition
    else if (lKeyData and $80000000) > 0 then
    begin
      details.setPressed(False);
    end;
  end;
end;

exports
  init, finish, create, destroy, action, show, hide, open, save, keyboard;

end.
